@@ -99,13 +99,13 @@ static SystemExceptionRec system_exceptions[] = {
     { "IDL:omg.org/CORBA/OBJECT_NOT_EXIST:1.0",
       "CORBA::OBJECT_NOT_EXIST",
       "Non-existent object, delete reference" },
-    { "IDL:omg.org/CORBA/TRANSACTION_REQUIRED",
+    { "IDL:omg.org/CORBA/TRANSACTION_REQUIRED:1.0",
       "CORBA::TRANSACTION_REQUIRED",
       "Transaction required" },
-    { "IDL:omg.org/CORBA/TRANSACTION_ROLLBACK",
-      "CORBA::TRANSACTION_ROLLBACK",
+    { "IDL:omg.org/CORBA/TRANSACTION_ROLLEDBACK:1.0",
+      "CORBA::TRANSACTION_ROLLEDBACK",
       "Transaction rolled back" },
-    { "IDL:omg.org/CORBA/INVALID_TRANSACTION",
+    { "IDL:omg.org/CORBA/INVALID_TRANSACTION:1.0",
       "CORBA::INVALID_TRANSACTION",
       "Invalid transaction" },
 };
